@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2018 Purism SPC
+ *               2024-2025 Phosh.mobi e.V.
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  *
@@ -16,8 +17,6 @@
 #include "util.h"
 #include "app-grid-button.h"
 
-#include <gio/gdesktopappinfo.h>
-
 /**
  * PhoshActivity:
  *
@@ -40,57 +39,60 @@ static guint signals[N_SIGNALS] = { 0 };
 
 enum {
   PROP_0,
+  PROP_APP_INFO,
   PROP_APP_ID,
   PROP_PARENT_APP_ID,
   PROP_MAXIMIZED,
   PROP_FULLSCREEN,
   PROP_WIN_WIDTH,
   PROP_WIN_HEIGHT,
+  PROP_HAS_THUMBNAIL,
   LAST_PROP,
 };
 static GParamSpec *props[LAST_PROP];
 
-typedef struct
-{
-  GtkWidget *swipe_bin;
-  GtkWidget *icon;
-  GtkWidget *box;
-  GtkWidget *revealer_close;
-  GtkWidget *revealer_unfullscreen;
-  GtkWidget *btn_close;
-  GtkWidget *btn_unfullscreen;
-  GtkWidget *preview;
-  GtkWidget *button;
+typedef struct {
+  GtkWidget       *swipe_bin;
+  GtkWidget       *icon;
+  GtkWidget       *box;
+  GtkWidget       *revealer_close;
+  GtkWidget       *revealer_unfullscreen;
+  GtkWidget       *btn_close;
+  GtkWidget       *btn_unfullscreen;
+  GtkWidget       *preview;
+  GtkWidget       *button;
+  GtkWidget       *spinner;
 
-  gboolean maximized;
-  gboolean fullscreen;
-  int win_width;
-  int win_height;
+  gboolean         maximized;
+  gboolean         fullscreen;
+  int              win_width;
+  int              win_height;
 
-  char *app_id;
-  char *parent_app_id;
+  char            *app_id;
+  char            *parent_app_id;
 
   cairo_surface_t *surface;
-  PhoshThumbnail *thumbnail;
+  PhoshThumbnail  *thumbnail;
 
-  gboolean hovering;
-  guint remove_timeout_id;
-  GtkAllocation allocation;
+  gboolean         hovering;
+  guint            remove_timeout_id;
+  GtkAllocation    allocation;
+
+  GAppInfo        *app_info;
 } PhoshActivityPrivate;
 
 
-struct _PhoshActivity
-{
+struct _PhoshActivity {
   GtkEventBox parent;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(PhoshActivity, phosh_activity, GTK_TYPE_EVENT_BOX)
+G_DEFINE_TYPE_WITH_PRIVATE (PhoshActivity, phosh_activity, GTK_TYPE_EVENT_BOX)
 
 
 static void
 set_fullscreen (PhoshActivity *self, gboolean fullscreen)
 {
-  PhoshActivityPrivate *priv = phosh_activity_get_instance_private(self);
+  PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
 
   priv->fullscreen = fullscreen;
   phosh_util_toggle_style_class (GTK_WIDGET (self), "phosh-fullscreen", priv->fullscreen);
@@ -100,87 +102,111 @@ set_fullscreen (PhoshActivity *self, gboolean fullscreen)
 
 
 static void
-phosh_activity_set_property (GObject *object,
-                             guint property_id,
+set_win_width (PhoshActivity *self, int width)
+{
+  PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
+
+  if (width == priv->win_width)
+    return;
+
+  priv->win_width = width;
+  gtk_widget_queue_resize (GTK_WIDGET (self));
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_WIN_WIDTH]);
+}
+
+
+static void
+set_win_height (PhoshActivity *self, int height)
+{
+  PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
+
+  if (height == priv->win_height)
+    return;
+
+  priv->win_height = height;
+  gtk_widget_queue_resize (GTK_WIDGET (self));
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_WIN_HEIGHT]);
+}
+
+
+static void
+phosh_activity_set_property (GObject      *object,
+                             guint         property_id,
                              const GValue *value,
-                             GParamSpec *pspec)
+                             GParamSpec   *pspec)
 {
   PhoshActivity *self = PHOSH_ACTIVITY (object);
-  PhoshActivityPrivate *priv = phosh_activity_get_instance_private(self);
-  int height, width;
+  PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
 
   switch (property_id) {
-    case PROP_APP_ID:
-      g_free (priv->app_id);
-      priv->app_id = g_value_dup_string (value);
-      g_object_notify_by_pspec (G_OBJECT (self), props[PROP_APP_ID]);
-      break;
-    case PROP_PARENT_APP_ID:
-      g_free (priv->parent_app_id);
-      priv->parent_app_id = g_value_dup_string (value);
-      g_object_notify_by_pspec (G_OBJECT (self), props[PROP_PARENT_APP_ID]);
-      break;
-    case PROP_MAXIMIZED:
-      priv->maximized = g_value_get_boolean (value);
-      phosh_util_toggle_style_class (GTK_WIDGET (self), "phosh-maximized", priv->maximized);
-      break;
-    case PROP_FULLSCREEN:
-      set_fullscreen (self, g_value_get_boolean (value));
-      break;
-    case PROP_WIN_WIDTH:
-      width = g_value_get_int (value);
-      if (width != priv->win_width) {
-        priv->win_width = width;
-        gtk_widget_queue_resize (GTK_WIDGET (self));
-        g_object_notify_by_pspec (G_OBJECT (self), props[PROP_WIN_WIDTH]);
-      }
-      break;
-    case PROP_WIN_HEIGHT:
-      height = g_value_get_int (value);
-      if (height != priv->win_height) {
-        priv->win_height = height;
-        gtk_widget_queue_resize (GTK_WIDGET (self));
-        g_object_notify_by_pspec (G_OBJECT (self), props[PROP_WIN_HEIGHT]);
-      }
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-      break;
+  case PROP_APP_INFO:
+    g_set_object (&priv->app_info, g_value_get_object (value));
+    break;
+  case PROP_APP_ID:
+    g_free (priv->app_id);
+    priv->app_id = g_value_dup_string (value);
+    break;
+  case PROP_PARENT_APP_ID:
+    g_free (priv->parent_app_id);
+    priv->parent_app_id = g_value_dup_string (value);
+    break;
+  case PROP_MAXIMIZED:
+    priv->maximized = g_value_get_boolean (value);
+    phosh_util_toggle_style_class (GTK_WIDGET (self), "phosh-maximized", priv->maximized);
+    break;
+  case PROP_FULLSCREEN:
+    set_fullscreen (self, g_value_get_boolean (value));
+    break;
+  case PROP_WIN_WIDTH:
+    set_win_width (self, g_value_get_int (value));
+    break;
+  case PROP_WIN_HEIGHT:
+    set_win_height (self, g_value_get_int (value));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    break;
   }
 }
 
 
 static void
-phosh_activity_get_property (GObject *object,
-                             guint property_id,
-                             GValue *value,
+phosh_activity_get_property (GObject    *object,
+                             guint       property_id,
+                             GValue     *value,
                              GParamSpec *pspec)
 {
   PhoshActivity *self = PHOSH_ACTIVITY (object);
-  PhoshActivityPrivate *priv = phosh_activity_get_instance_private(self);
+  PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
 
   switch (property_id) {
-    case PROP_APP_ID:
-      g_value_set_string (value, priv->app_id);
-      break;
-    case PROP_PARENT_APP_ID:
-      g_value_set_string (value, priv->parent_app_id);
-      break;
-    case PROP_MAXIMIZED:
-      g_value_set_boolean (value, priv->maximized);
-      break;
-    case PROP_FULLSCREEN:
-      g_value_set_boolean (value, priv->fullscreen);
-      break;
-    case PROP_WIN_WIDTH:
-      g_value_set_int (value, priv->win_width);
-      break;
-    case PROP_WIN_HEIGHT:
-      g_value_set_int (value, priv->win_height);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-      break;
+  case PROP_APP_INFO:
+    g_value_set_object (value, phosh_activity_get_app_info (self));
+    break;
+  case PROP_APP_ID:
+    g_value_set_string (value, phosh_activity_get_app_id (self));
+    break;
+  case PROP_PARENT_APP_ID:
+    g_value_set_string (value, priv->parent_app_id);
+    break;
+  case PROP_MAXIMIZED:
+    g_value_set_boolean (value, priv->maximized);
+    break;
+  case PROP_FULLSCREEN:
+    g_value_set_boolean (value, priv->fullscreen);
+    break;
+  case PROP_WIN_WIDTH:
+    g_value_set_int (value, priv->win_width);
+    break;
+  case PROP_WIN_HEIGHT:
+    g_value_set_int (value, priv->win_height);
+    break;
+  case PROP_HAS_THUMBNAIL:
+    g_value_set_boolean (value, phosh_activity_get_has_thumbnail (self));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    break;
   }
 }
 
@@ -209,16 +235,15 @@ on_unfullscreen_clicked (PhoshActivity *self)
 }
 
 
-static gboolean
-remove_timeout_cb (PhoshActivity *self)
+static void
+on_remove_timeout (gpointer data)
 {
+  PhoshActivity *self = PHOSH_ACTIVITY (data);
   PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
 
   phosh_swipe_away_bin_undo (PHOSH_SWIPE_AWAY_BIN (priv->swipe_bin));
 
   priv->remove_timeout_id = 0;
-
-  return G_SOURCE_REMOVE;
 }
 
 
@@ -230,8 +255,7 @@ removed_cb (PhoshActivity *self)
   if (priv->remove_timeout_id)
     g_source_remove (priv->remove_timeout_id);
 
-  priv->remove_timeout_id =
-    g_timeout_add_seconds (1, (GSourceFunc) remove_timeout_cb, self);
+  priv->remove_timeout_id = g_timeout_add_seconds_once (1, on_remove_timeout, self);
   g_source_set_name_by_id (priv->remove_timeout_id, "[phosh] remove_timeout_id");
 
   g_signal_emit (self, signals[CLOSED], 0);
@@ -255,18 +279,28 @@ get_scale (PhoshActivity *self)
   image_width = cairo_image_surface_get_width (priv->surface);
   image_height = cairo_image_surface_get_height (priv->surface);
 
-  scale = width / (float)image_width;
-
-  if (height / (float)image_height < scale)
-    scale = height / (float)image_height;
+  scale = MIN (width / (float)image_width, height / (float)image_height);
 
   return scale;
 }
 
+
+static void
+draw_rounded_rect (cairo_t *ctx, double x, double y,  double width, double height, double radius)
+{
+  cairo_new_sub_path (ctx);
+  cairo_arc (ctx, x + width - radius, y + radius, radius, -0.5 * M_PI, 0);
+  cairo_arc (ctx, x + width - radius, y + height - radius, radius, 0, 0.5 * M_PI);
+  cairo_arc (ctx, x + radius, y + height - radius, radius, 0.5 * M_PI, M_PI);
+  cairo_arc (ctx, x + radius, y + radius, radius, M_PI, 1.5 * M_PI);
+  cairo_close_path (ctx);
+}
+
+
 static gboolean
 draw_cb (PhoshActivity *self, cairo_t *cairo, GtkDrawingArea *area)
 {
-  int width, height, image_width, image_height, x, y = 0;
+  int width, height, image_width, image_height, border_radius, x, y = 0;
   float scale;
   PhoshActivityPrivate *priv;
   GtkStyleContext *context;
@@ -275,24 +309,29 @@ draw_cb (PhoshActivity *self, cairo_t *cairo, GtkDrawingArea *area)
   g_return_val_if_fail (GTK_IS_DRAWING_AREA (area), FALSE);
 
   priv = phosh_activity_get_instance_private (self);
-  if (!priv->surface)
-    return FALSE;
 
   context = gtk_widget_get_style_context (GTK_WIDGET (area));
   width = gtk_widget_get_allocated_width (GTK_WIDGET (area));
   height = gtk_widget_get_allocated_height (GTK_WIDGET (area));
 
+  gtk_render_background (context, cairo, 0, 0, width, height);
+
+  if (!priv->surface)
+    return FALSE;
+
   image_width = cairo_image_surface_get_width (priv->surface);
   image_height = cairo_image_surface_get_height (priv->surface);
 
-  gtk_render_background (context, cairo, 0, 0, width, height);
-
   scale = get_scale (self);
   cairo_scale (cairo, scale, scale);
 
   x = (width - image_width * scale) / 2.0 / scale;
 
-  cairo_rectangle (cairo, x, y, image_width, image_height);
+  gtk_style_context_get (context,
+                         gtk_style_context_get_state (context),
+                         GTK_STYLE_PROPERTY_BORDER_RADIUS, &border_radius,
+                         NULL);
+  draw_rounded_rect (cairo, x, y, image_width, image_height, border_radius);
   cairo_set_source_surface (cairo, priv->surface, x, y);
   cairo_fill (cairo);
 
@@ -319,26 +358,30 @@ phosh_activity_constructed (GObject *object)
 {
   PhoshActivity *self = PHOSH_ACTIVITY (object);
   PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
-  g_autoptr (GDesktopAppInfo) app_info = NULL;
   GIcon *icon = NULL;
 
-  app_info = phosh_get_desktop_app_info_for_app_id (priv->app_id);
-  if (app_info)
-    icon = g_app_info_get_icon (G_APP_INFO (app_info));
+  if (priv->app_info == NULL)
+    priv->app_info = G_APP_INFO (phosh_get_desktop_app_info_for_app_id (priv->app_id));
+
+  if (priv->app_info)
+    icon = g_app_info_get_icon (priv->app_info);
 
   if (!icon && priv->parent_app_id) {
-    app_info = phosh_get_desktop_app_info_for_app_id (priv->parent_app_id);
-    if (app_info)
-      icon = g_app_info_get_icon (G_APP_INFO (app_info));
+    priv->app_info = G_APP_INFO (phosh_get_desktop_app_info_for_app_id (priv->parent_app_id));
+    if (priv->app_info)
+      icon = g_app_info_get_icon (priv->app_info);
   }
 
   if (icon) {
     gtk_image_set_from_gicon (GTK_IMAGE (priv->icon), icon, ACTIVITY_ICON_SIZE);
   } else {
-    gtk_image_set_from_icon_name (GTK_IMAGE (priv->icon), PHOSH_APP_UNKNOWN_ICON, ACTIVITY_ICON_SIZE);
-    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (self)), "phosh-empty");
+    gtk_image_set_from_icon_name (GTK_IMAGE (priv->icon),
+                                  PHOSH_APP_UNKNOWN_ICON,
+                                  ACTIVITY_ICON_SIZE);
   }
 
+  phosh_util_toggle_style_class (GTK_WIDGET (self), "phosh-empty", TRUE);
+
   G_OBJECT_CLASS (phosh_activity_parent_class)->constructed (object);
 }
 
@@ -351,6 +394,7 @@ phosh_activity_dispose (GObject *object)
 
   g_clear_pointer (&priv->surface, cairo_surface_destroy);
   g_clear_object (&priv->thumbnail);
+  g_clear_object (&priv->app_info);
 
   if (priv->remove_timeout_id) {
     g_source_remove (priv->remove_timeout_id);
@@ -440,7 +484,8 @@ phosh_activity_get_preferred_width_for_height (GtkWidget *widget,
   margin_bottom = gtk_widget_get_margin_bottom (priv->preview);
 
   aspect_ratio = (double) priv->win_width / priv->win_height;
-  size = MAX (smallest, (height - margin_top - margin_bottom) * aspect_ratio) + margin_start + margin_end;
+  size = MAX (smallest,
+              (height - margin_top - margin_bottom) * aspect_ratio) + margin_start + margin_end;
 
   if (min)
     *min = size;
@@ -464,7 +509,10 @@ static void
 set_hovering (PhoshActivity *self,
               gboolean       hovering)
 {
-  PhoshActivityPrivate *priv = phosh_activity_get_instance_private(self);
+  PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
+
+  if (!phosh_activity_get_has_thumbnail (self))
+    return;
 
   if (hovering == priv->hovering)
     return;
@@ -577,15 +625,24 @@ phosh_activity_class_init (PhoshActivityClass *klass)
   widget_class->key_press_event = phosh_activity_key_press_event;
   widget_class->unmap = phosh_activity_unmap;
 
+  /**
+   * PhoshActivity:app-info:
+   *
+   * The app-info of the activity
+   */
+  props[PROP_APP_INFO] =
+    g_param_spec_object ("app-info", "", "",
+                         G_TYPE_APP_INFO,
+                         G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  /**
+   * PhoshActivity:app-id:
+   *
+   * The app-id of the activity
+   */
   props[PROP_APP_ID] =
-    g_param_spec_string (
-      "app-id",
-      "app-id",
-      "The application id",
-      "",
-      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE |
-      G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
+    g_param_spec_string ("app-id", "", "",
+                         "",
+                         G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   /**
    * PhoshActivity:parent-app-id:
    *
@@ -594,8 +651,7 @@ phosh_activity_class_init (PhoshActivityClass *klass)
   props[PROP_PARENT_APP_ID] =
     g_param_spec_string ("parent-app-id", "", "",
                          NULL,
-                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE |
-                         G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   /**
    * PhoshActivity:maximized:
    *
@@ -621,9 +677,7 @@ phosh_activity_class_init (PhoshActivityClass *klass)
    */
   props[PROP_WIN_WIDTH] =
     g_param_spec_int ("win-width", "", "",
-                      0,
-                      G_MAXINT,
-                      300,
+                      0, G_MAXINT, 300,
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
   /**
    * PhoshActivity:win-height:
@@ -632,10 +686,17 @@ phosh_activity_class_init (PhoshActivityClass *klass)
    */
   props[PROP_WIN_HEIGHT] =
     g_param_spec_int ("win-height", "", "",
-                      0,
-                      G_MAXINT,
-                      300,
+                      0, G_MAXINT, 300,
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+  /**
+   * PhoshActivity:has-thumbnail:
+   *
+   * Whether the activity is backed by a thumbnail
+   */
+  props[PROP_HAS_THUMBNAIL] =
+    g_param_spec_boolean ("has-thumbnail", "", "",
+                          FALSE,
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
@@ -688,6 +749,7 @@ phosh_activity_class_init (PhoshActivityClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, PhoshActivity, box);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshActivity, revealer_close);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshActivity, revealer_unfullscreen);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshActivity, spinner);
   gtk_widget_class_bind_template_callback (widget_class, clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, closed_cb);
   gtk_widget_class_bind_template_callback (widget_class, draw_cb);
@@ -728,9 +790,22 @@ phosh_activity_get_app_id (PhoshActivity *self)
   g_return_val_if_fail (PHOSH_IS_ACTIVITY (self), NULL);
   priv = phosh_activity_get_instance_private (self);
 
-  return priv->app_id;
+  if (priv->app_id)
+    return priv->app_id;
+
+  if (priv->app_info)
+    return g_app_info_get_id (priv->app_info);
+
+  return NULL;
 }
 
+/**
+ * phosh_activity_set_thumbnail:
+ * @self: the activity
+ * @thumbnail:(transfer full): the thumbnail
+ *
+ * Sets the given thumbnail
+ */
 void
 phosh_activity_set_thumbnail (PhoshActivity *self, PhoshThumbnail *thumbnail)
 {
@@ -738,19 +813,22 @@ phosh_activity_set_thumbnail (PhoshActivity *self, PhoshThumbnail *thumbnail)
   gpointer data;
   guint w, width, height, stride, margin;
   float scale;
+  gboolean has_thumbnail;
 
   g_return_if_fail (PHOSH_IS_ACTIVITY (self));
   priv = phosh_activity_get_instance_private (self);
 
-  g_clear_pointer (&priv->surface, cairo_surface_destroy);
+  has_thumbnail = !!priv->thumbnail;
   g_clear_object (&priv->thumbnail);
+  priv->thumbnail = thumbnail;
 
   data = phosh_thumbnail_get_image (thumbnail);
   phosh_thumbnail_get_size (thumbnail, &width, &height, &stride);
 
-  priv->surface = cairo_image_surface_create_for_data (
-      data, CAIRO_FORMAT_ARGB32, width, height, stride);
-  priv->thumbnail = thumbnail;
+  g_clear_pointer (&priv->surface, cairo_surface_destroy);
+  priv->surface = cairo_image_surface_create_for_data (data,
+                                                       CAIRO_FORMAT_ARGB32,
+                                                       width, height, stride);
 
   phosh_util_toggle_style_class (GTK_WIDGET (self), "phosh-empty", FALSE);
 
@@ -762,6 +840,9 @@ phosh_activity_set_thumbnail (PhoshActivity *self, PhoshThumbnail *thumbnail)
   gtk_widget_set_margin_end (priv->btn_close, margin);
 
   gtk_widget_queue_draw (GTK_WIDGET (self));
+
+  if (has_thumbnail != !!priv->thumbnail)
+    g_object_notify_by_pspec (G_OBJECT (self), props[PROP_HAS_THUMBNAIL]);
 }
 
 void
@@ -775,3 +856,25 @@ phosh_activity_get_thumbnail_allocation (PhoshActivity *self, GtkAllocation *all
 
   *allocation = priv->allocation;
 }
+
+
+gboolean
+phosh_activity_get_has_thumbnail (PhoshActivity *self)
+{
+  PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
+
+  g_return_val_if_fail (PHOSH_IS_ACTIVITY (self), FALSE);
+
+  return !!priv->thumbnail;
+}
+
+
+GAppInfo *
+phosh_activity_get_app_info (PhoshActivity *self)
+{
+  PhoshActivityPrivate *priv = phosh_activity_get_instance_private (self);
+
+  g_return_val_if_fail (PHOSH_IS_ACTIVITY (self), NULL);
+
+  return priv->app_info;
+}
