@@ -13,6 +13,8 @@
 
 #include "phosh-config.h"
 
+#include "accounts-dbus.h"
+#include "accounts-user-dbus.h"
 #include "polkit-auth-prompt.h"
 
 #define GCR_API_SUBJECT_TO_CHANGE
@@ -22,6 +24,7 @@
 #include <polkitagent/polkitagent.h>
 
 #include <glib/gi18n.h>
+#include <handy.h>
 
 /**
  * PhoshPolkitAuthPrompt:
@@ -47,35 +50,38 @@ enum {
   DONE,
   N_SIGNALS
 };
-static guint signals[N_SIGNALS] = { 0 };
+static guint signals[N_SIGNALS];
 
-struct _PhoshPolkitAuthPrompt
-{
+struct _PhoshPolkitAuthPrompt {
   PhoshSystemModalDialog parent;
 
-  GtkWidget *lbl_message;
-  GtkWidget *lbl_user_name;
-  GtkWidget *lbl_password;
-  GtkWidget *lbl_info;
-  GtkWidget *img_icon;
-  GtkWidget *btn_authenticate;
-  GtkWidget *spinner_authenticate;
-  GtkWidget *btn_cancel;
-  GtkWidget *entry_password;
+  GtkWidget      *lbl_message;
+  GtkWidget      *lbl_user_name;
+  GtkWidget      *lbl_password;
+  GtkWidget      *lbl_info;
+  GtkWidget      *img_icon;
+  GtkWidget      *btn_authenticate;
+  GtkWidget      *spinner_authenticate;
+  GtkWidget      *btn_cancel;
+  GtkWidget      *entry_password;
   GtkEntryBuffer *password_buffer;
 
-  char  *action_id;
-  char  *message;
-  char  *icon_name;
-  char  *cookie;
-  GStrv  user_names;
+  char *action_id;
+  char *message;
+  char *icon_name;
+  char *cookie;
+  GStrv user_names;
 
-  PolkitIdentity *identity;
+  const char *user_name;
+  GCancellable *cancel;
+
+  PolkitIdentity     *identity;
   PolkitAgentSession *session;
 
   gboolean done_emitted;
 };
-G_DEFINE_TYPE(PhoshPolkitAuthPrompt, phosh_polkit_auth_prompt, PHOSH_TYPE_SYSTEM_MODAL_DIALOG);
+
+G_DEFINE_TYPE (PhoshPolkitAuthPrompt, phosh_polkit_auth_prompt, PHOSH_TYPE_SYSTEM_MODAL_DIALOG);
 
 
 static void phosh_polkit_auth_prompt_initiate (PhoshPolkitAuthPrompt *self);
@@ -137,13 +143,98 @@ set_icon_name (PhoshPolkitAuthPrompt *self, const char *icon_name)
 
   g_clear_pointer (&self->icon_name, g_free);
   self->icon_name = g_strdup (icon_name);
-  gtk_image_set_from_icon_name (GTK_IMAGE (self->img_icon),
-                                (icon_name && strlen(icon_name)) ? icon_name : "dialog-password-symbolic",
-                                GTK_ICON_SIZE_DIALOG);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ICON_NAME]);
 }
 
+
+static void
+on_accounts_user_proxy_ready (GObject *source, GAsyncResult *result, gpointer data)
+{
+  PhoshPolkitAuthPrompt *self = data;
+  g_autoptr (GError) error = NULL;
+  g_autoptr (PhoshDBusAccountsUser) user = NULL;
+  const char *real_name = NULL;
+  const char *icon_path = NULL;
+  g_autoptr (GFile) icon_file = NULL;
+  g_autoptr (GIcon) icon = NULL;
+
+  user = phosh_dbus_accounts_user_proxy_new_finish (result, &error);
+  if (error) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_debug ("Skipping create user proxy: %s", error->message);
+    else
+      g_warning ("Failed to create user proxy %s: %s", self->user_name, error->message);
+    return;
+  }
+  g_debug ("Created proxy for user name: %s", self->user_name);
+
+  real_name = phosh_dbus_accounts_user_get_real_name (user);
+  hdy_avatar_set_text (HDY_AVATAR (self->img_icon), real_name);
+  gtk_label_set_text (GTK_LABEL (self->lbl_user_name), real_name);
+
+  icon_path = phosh_dbus_accounts_user_get_icon_file (user);
+  icon_file = g_file_new_for_path (icon_path);
+  icon = g_file_icon_new (icon_file);
+  hdy_avatar_set_loadable_icon (HDY_AVATAR (self->img_icon), G_LOADABLE_ICON (icon));
+}
+
+
+static void
+on_find_user_by_name_ready (GObject *source, GAsyncResult *result, gpointer data)
+{
+  g_autoptr (PhoshDBusAccounts) accounts = PHOSH_DBUS_ACCOUNTS (source);
+  PhoshPolkitAuthPrompt *self = data;
+  g_autoptr (GError) error = NULL;
+  g_autofree char *user_path = NULL;
+
+  phosh_dbus_accounts_call_find_user_by_name_finish (accounts, &user_path, result, &error);
+  if (error) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_debug ("Skipping find user: %s", error->message);
+    else
+      g_warning ("Failed to find user %s: %s", self->user_name, error->message);
+    return;
+  }
+  g_debug ("Found user for user name: %s at %s", self->user_name, user_path);
+
+  g_debug ("Creating proxy for user name: %s", self->user_name);
+  phosh_dbus_accounts_user_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
+                                              G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+                                              "org.freedesktop.Accounts",
+                                              user_path,
+                                              self->cancel,
+                                              on_accounts_user_proxy_ready,
+                                              self);
+}
+
+
+static void
+on_accounts_proxy_ready (GObject *source, GAsyncResult *result, gpointer data)
+{
+  PhoshDBusAccounts *accounts;
+  PhoshPolkitAuthPrompt *self = data;
+  g_autoptr (GError) error = NULL;
+
+  accounts = phosh_dbus_accounts_proxy_new_finish (result, &error);
+  if (error) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_debug ("Skipping create Accounts proxy: %s", error->message);
+    else
+      g_warning ("Failed to create Accounts proxy: %s", error->message);
+    return;
+  }
+  g_debug ("Created Accounts proxy for user name: %s", self->user_name);
+
+  g_debug ("Finding user for user name: %s", self->user_name);
+  phosh_dbus_accounts_call_find_user_by_name (accounts,
+                                              self->user_name,
+                                              self->cancel,
+                                              on_find_user_by_name_ready,
+                                              self);
+}
+
+
 static void
 set_user_names (PhoshPolkitAuthPrompt *self, const GStrv user_names)
 {
@@ -167,84 +258,97 @@ set_user_names (PhoshPolkitAuthPrompt *self, const GStrv user_names)
     user_name = self->user_names[0];
   } else {
     g_debug ("Received %d user names, only using one", len);
-    user_name = g_get_user_name();
+    user_name = g_get_user_name ();
 
     if (!g_strv_contains ((const char *const *)user_names, user_name))
       user_name = "root";
 
     if (!g_strv_contains ((const char *const *)user_names, user_name))
       user_name = user_names[0];
-  };
+  }
 
   self->identity = polkit_unix_user_new_for_name (user_name, &err);
   if (!self->identity) {
-    g_warning("Failed to create identity: %s", err->message);
+    g_warning ("Failed to create identity: %s", err->message);
     g_clear_error (&err);
     return;
   }
 
+  hdy_avatar_set_text (HDY_AVATAR (self->img_icon), user_name);
   gtk_label_set_text (GTK_LABEL (self->lbl_user_name), user_name);
+
+  self->user_name = user_name;
+  g_debug ("Creating Accounts proxy for user name: %s", self->user_name);
+  phosh_dbus_accounts_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
+                                         (G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
+                                          G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS),
+                                         "org.freedesktop.Accounts",
+                                         "/org/freedesktop/Accounts",
+                                         self->cancel,
+                                         on_accounts_proxy_ready,
+                                         self);
+
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_USER_NAMES]);
 }
 
 
 static void
-phosh_polkit_auth_prompt_set_property (GObject      *obj,
-                                       guint         prop_id,
+phosh_polkit_auth_prompt_set_property (GObject      *object,
+                                       guint         property_id,
                                        const GValue *value,
                                        GParamSpec   *pspec)
 {
-  PhoshPolkitAuthPrompt *self = PHOSH_POLKIT_AUTH_PROMPT (obj);
+  PhoshPolkitAuthPrompt *self = PHOSH_POLKIT_AUTH_PROMPT (object);
 
-  switch (prop_id) {
+  switch (property_id) {
   case PROP_ACTION_ID:
-    set_action_id (self, g_value_get_string(value));
+    set_action_id (self, g_value_get_string (value));
     break;
   case PROP_COOKIE:
-    set_cookie (self, g_value_get_string(value));
+    set_cookie (self, g_value_get_string (value));
     break;
   case PROP_MESSAGE:
-    set_message (self, g_value_get_string(value));
+    set_message (self, g_value_get_string (value));
     break;
   case PROP_ICON_NAME:
-    set_icon_name (self, g_value_get_string(value));
+    set_icon_name (self, g_value_get_string (value));
     break;
   case PROP_USER_NAMES:
-    set_user_names (self, g_value_get_boxed(value));
+    set_user_names (self, g_value_get_boxed (value));
     break;
   default:
-    G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
   }
 }
 
 
 static void
-phosh_polkit_auth_prompt_get_property (GObject    *obj,
-                                       guint       prop_id,
+phosh_polkit_auth_prompt_get_property (GObject    *object,
+                                       guint       property_id,
                                        GValue     *value,
                                        GParamSpec *pspec)
 {
-  PhoshPolkitAuthPrompt *self = PHOSH_POLKIT_AUTH_PROMPT (obj);
+  PhoshPolkitAuthPrompt *self = PHOSH_POLKIT_AUTH_PROMPT (object);
 
-  switch (prop_id) {
+  switch (property_id) {
   case PROP_ACTION_ID:
-    g_value_set_string (value, self->action_id?: "");
+    g_value_set_string (value, self->action_id ?: "");
     break;
   case PROP_COOKIE:
-    g_value_set_string (value, self->cookie?: "");
+    g_value_set_string (value, self->cookie ?: "");
     break;
   case PROP_MESSAGE:
-    g_value_set_string (value, self->message?: "");
+    g_value_set_string (value, self->message ?: "");
     break;
   case PROP_ICON_NAME:
-    g_value_set_string (value, self->icon_name?: "");
+    g_value_set_string (value, self->icon_name ?: "");
     break;
   case PROP_USER_NAMES:
-    g_value_set_boxed (value, self->user_names ? g_strdupv(self->user_names) : NULL);
+    g_value_set_boxed (value, self->user_names ? g_strdupv (self->user_names) : NULL);
     break;
   default:
-    G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
   }
 }
@@ -268,7 +372,7 @@ on_auth_session_request (PhoshPolkitAuthPrompt *self,
                          gboolean               echo_on,
                          gpointer               user_data)
 {
-  g_debug("Request: %s, echo: %d", request, echo_on);
+  g_debug ("Request: %s, echo: %d", request, echo_on);
 
   if (self->done_emitted)
     return;
@@ -277,7 +381,7 @@ on_auth_session_request (PhoshPolkitAuthPrompt *self,
   if (!g_strcmp0 (request, "Password: ") || !g_strcmp0 (request, "Password:"))
     request = _("Password:");
 
-  gtk_label_set_text(GTK_LABEL (self->lbl_password), request);
+  gtk_label_set_text (GTK_LABEL (self->lbl_password), request);
   gtk_entry_set_visibility (GTK_ENTRY (self->entry_password), echo_on);
   gtk_entry_set_text (GTK_ENTRY (self->entry_password), "");
   gtk_widget_grab_focus (self->entry_password);
@@ -316,11 +420,11 @@ on_auth_session_completed (PhoshPolkitAuthPrompt *self,
     return;
 
   if (gained_auth) {
-      emit_done (self, FALSE);
+    emit_done (self, FALSE);
   } else {
     const char *info = gtk_label_get_text (GTK_LABEL (self->lbl_info));
 
-    if (info == NULL || !g_strcmp0(info, "")) {
+    if (info == NULL || !g_strcmp0 (info, "")) {
       gtk_label_set_text (GTK_LABEL (self->lbl_info),
                           _("Sorry, that didn’t work. Please try again."));
     }
@@ -392,21 +496,24 @@ on_btn_authenticate_clicked (PhoshPolkitAuthPrompt *self, GtkButton *btn)
 
 
 static void
-phosh_polkit_auth_prompt_dispose (GObject *obj)
+phosh_polkit_auth_prompt_dispose (GObject *object)
 {
-  PhoshPolkitAuthPrompt *self = PHOSH_POLKIT_AUTH_PROMPT (obj);
+  PhoshPolkitAuthPrompt *self = PHOSH_POLKIT_AUTH_PROMPT (object);
 
+  g_cancellable_cancel (self->cancel);
+  g_clear_object (&self->cancel);
+  self->user_name = NULL;
   g_clear_object (&self->identity);
   g_clear_object (&self->session);
 
-  G_OBJECT_CLASS (phosh_polkit_auth_prompt_parent_class)->dispose (obj);
+  G_OBJECT_CLASS (phosh_polkit_auth_prompt_parent_class)->dispose (object);
 }
 
 
 static void
-phosh_polkit_auth_prompt_finalize (GObject *obj)
+phosh_polkit_auth_prompt_finalize (GObject *object)
 {
-  PhoshPolkitAuthPrompt *self = PHOSH_POLKIT_AUTH_PROMPT (obj);
+  PhoshPolkitAuthPrompt *self = PHOSH_POLKIT_AUTH_PROMPT (object);
 
   g_free (self->message);
   g_free (self->icon_name);
@@ -414,7 +521,7 @@ phosh_polkit_auth_prompt_finalize (GObject *obj)
   g_free (self->cookie);
   g_strfreev (self->user_names);
 
-  G_OBJECT_CLASS (phosh_polkit_auth_prompt_parent_class)->finalize (obj);
+  G_OBJECT_CLASS (phosh_polkit_auth_prompt_parent_class)->finalize (object);
 }
 
 
@@ -425,6 +532,8 @@ phosh_polkit_auth_prompt_constructed (GObject *object)
 
   G_OBJECT_CLASS (phosh_polkit_auth_prompt_parent_class)->constructed (object);
 
+  self->cancel = g_cancellable_new ();
+
   self->password_buffer = gcr_secure_entry_buffer_new ();
   gtk_entry_set_buffer (GTK_ENTRY (self->entry_password),
                         GTK_ENTRY_BUFFER (self->password_buffer));
@@ -436,7 +545,7 @@ phosh_polkit_auth_prompt_constructed (GObject *object)
 static void
 phosh_polkit_auth_prompt_class_init (PhoshPolkitAuthPromptClass *klass)
 {
-  GObjectClass *object_class = (GObjectClass *)klass;
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->get_property = phosh_polkit_auth_prompt_get_property;
@@ -445,39 +554,54 @@ phosh_polkit_auth_prompt_class_init (PhoshPolkitAuthPromptClass *klass)
   object_class->dispose = phosh_polkit_auth_prompt_dispose;
   object_class->finalize = phosh_polkit_auth_prompt_finalize;
 
+  /**
+   * PolkitAuthPrompt:action-id:
+   *
+   * The prompt's action id
+   */
   props[PROP_ACTION_ID] =
-    g_param_spec_string ("action-id",
-                         "Action ID",
-                         "The prompt's action id",
+    g_param_spec_string ("action-id", "", "",
                          "",
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_CONSTRUCT_ONLY);
+  /**
+   * PolkitAuthPrompt:cookie:
+   *
+   * The prompt's cookie
+   */
   props[PROP_COOKIE] =
-    g_param_spec_string ("cookie",
-                         "Cookie",
-                         "The prompt's cookie",
+    g_param_spec_string ("cookie", "", "",
                          "",
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_CONSTRUCT_ONLY);
+  /**
+   * PolkitAuthPrompt:message:
+   *
+   * The prompt's message
+   */
   props[PROP_MESSAGE] =
-    g_param_spec_string ("message",
-                         "Message",
-                         "The prompt's message",
+    g_param_spec_string ("message", "", "",
                          "",
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_CONSTRUCT_ONLY);
+  /**
+   * PolkitAuthPrompt:icon-name:
+   *
+   * The prompt's icon name
+   */
   props[PROP_ICON_NAME] =
-    g_param_spec_string ("icon-name",
-                         "Icon name",
-                         "The prompt's icon name",
+    g_param_spec_string ("icon-name", "", "",
                          "",
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_CONSTRUCT_ONLY);
+  /**
+   * PolkitAuthPrompt:user-names:
+   *
+   * The user names to authenticate as
+   */
   props[PROP_USER_NAMES] =
-    g_param_spec_boxed ("user-names",
-                        "User names",
-                        "The user name's to authenticate as",
+    g_param_spec_boxed ("user-names", "", "",
                         G_TYPE_STRV,
                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_CONSTRUCT_ONLY);
   g_object_class_install_properties (object_class, PROP_LAST_PROP, props);
 
- /**
+  /**
    * PhoshPolkitAuthPrompt::done:
    * @self: The #PhoshPolkitAuthPrompt instance.
    * @cancelled: Whether the dialog was cancelled
@@ -485,23 +609,27 @@ phosh_polkit_auth_prompt_class_init (PhoshPolkitAuthPromptClass *klass)
    * This signal is emitted when the prompt can be closed. The cancelled
    * argument indicates whether the prompt was cancelled.
    */
-  signals[DONE] = g_signal_new (
-    "done", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-    NULL, G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+  signals[DONE] = g_signal_new ("done",
+                                G_TYPE_FROM_CLASS (klass),
+                                G_SIGNAL_RUN_LAST,
+                                0, NULL, NULL, NULL,
+                                G_TYPE_NONE,
+                                1, G_TYPE_BOOLEAN);
 
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/mobi/phosh/ui/polkit-auth-prompt.ui");
-  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, lbl_message);
-  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, lbl_user_name);
-  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, lbl_password);
-  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, lbl_info);
-  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, img_icon);
   gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, btn_authenticate);
   gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, btn_cancel);
   gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, entry_password);
+  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, img_icon);
+  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, lbl_info);
+  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, lbl_password);
+  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, lbl_user_name);
   gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, spinner_authenticate);
-  gtk_widget_class_bind_template_callback (widget_class, on_dialog_canceled);
+  gtk_widget_class_bind_template_child (widget_class, PhoshPolkitAuthPrompt, lbl_message);
+
   gtk_widget_class_bind_template_callback (widget_class, on_btn_authenticate_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, on_dialog_canceled);
 }
 
 
@@ -517,10 +645,9 @@ phosh_polkit_auth_prompt_new (const char *action_id,
                               const char *message,
                               const char *icon_name,
                               const char *cookie,
-                              GStrv user_names)
+                              GStrv       user_names)
 {
   return g_object_new (PHOSH_TYPE_POLKIT_AUTH_PROMPT,
-                       /* polkit prompt */
                        "action-id", action_id,
                        "cookie", cookie,
                        "message", message,
