@@ -1,12 +1,12 @@
 /*
- * Copyright (C) 2023 Guido Günther <agx@sigxcpu.org>
+ * Copyright (C) 2023 Phosh.mobi e.V.
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 #pragma once
 
-#include "audio-device.h"
+#include "audio/audio-device.h"
 
 #include <gtk/gtk.h>
 
