@@ -25,3 +25,6 @@ struct _PhoshSplashClass {
 
 
 GtkWidget *phosh_splash_new (GDesktopAppInfo *info, gboolean prefer_dark);
+void phosh_splash_hide (PhoshSplash *self);
+void phosh_splash_lower (PhoshSplash *self);
+void phosh_splash_raise (PhoshSplash *self);
