@@ -8,9 +8,10 @@
 
 #include <gtk/gtk.h>
 
+#include "app-tracker.h"
 #include "toplevel.h"
 
-#define PHOSH_TYPE_TOPLEVEL_MANAGER (phosh_toplevel_manager_get_type())
+#define PHOSH_TYPE_TOPLEVEL_MANAGER (phosh_toplevel_manager_get_type ())
 
 G_DECLARE_FINAL_TYPE (PhoshToplevelManager,
                       phosh_toplevel_manager,
@@ -24,3 +25,5 @@ PhoshToplevel        *phosh_toplevel_manager_get_toplevel        (PhoshToplevelM
 guint                 phosh_toplevel_manager_get_num_toplevels   (PhoshToplevelManager *self);
 PhoshToplevel        *phosh_toplevel_manager_get_parent          (PhoshToplevelManager *self,
                                                                   PhoshToplevel        *toplevel);
+void                  phosh_toplevel_manager_set_app_tracker (PhoshToplevelManager *self,
+                                                              PhoshAppTracker      *app_tracker);
