@@ -76,7 +76,7 @@ test_phosh_mount_notification_new (void)
   g_assert_true (g_strcmp0 (g_themed_icon_get_names (G_THEMED_ICON (icon))[0],
                             "applications-system-symbolic") == 0 ||
                  g_strcmp0 (g_themed_icon_get_names (G_THEMED_ICON (icon))[0],
-                            "sm.puri.Phosh") == 0);
+                            "mobi.phosh.Shell") == 0);
 
   icon = phosh_notification_get_image (PHOSH_NOTIFICATION (mn));
   g_assert_true (G_IS_THEMED_ICON (icon));
